"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_debounce_1 = __importDefault(require("lodash.debounce"));
var react_1 = require("react");
/**
 * @typeparam T - The type of items to search.
 */
var Searchable = /** @class */ (function (_super) {
    __extends(Searchable, _super);
    function Searchable(props) {
        var _this = _super.call(this, props) || this;
        /**
         * Change event handler for updating value in state.
         *
         * @param event - An instance of ChangeEvent
         */
        _this.handleChange = function (event) {
            return _this.setState({ value: event.target.value });
        };
        var value = props.initialValue, predicate = props.predicate, items = props.items, shouldDebounce = props.debounce;
        _this.state = {
            items: value !== '' ? Searchable.filter(items, value, predicate) : items,
            value: value,
        };
        if (shouldDebounce) {
            _this.filterAndSetState = lodash_debounce_1.default(_this.filterAndSetState, _this.props.debounceDuration);
        }
        return _this;
    }
    /**
     * Filters an array of items based on a search value and a filtering predicate.
     *
     * @typeparam T - The type of items.
     * @param items - The array of items to be filtered.
     * @param value - A search string to be passed to the predicate.
     * @param predicate - A filtering predicate based on an item and the search value.
     * @returns A filtered array of items.
     */
    Searchable.filter = function (items, value, predicate) {
        return items.filter(function (item) { return predicate(item, value); });
    };
    /**
     * Call filtering method if [[IState.value]] has changed.
     */
    Searchable.prototype.componentDidUpdate = function (prevProps, prevState) {
        var value = this.state.value;
        if (value !== prevState.value) {
            var items = this.props.items;
            this.filterAndSetState(items, value);
        }
    };
    Searchable.prototype.render = function () {
        var _a = this.state, items = _a.items, value = _a.value;
        var _b = this.props, render = _b.render, children = _b.children;
        var renderParams = { items: items, value: value, handleChange: this.handleChange };
        if (render) {
            return render(renderParams);
        }
        else if (children) {
            return children(renderParams);
        }
        else {
            return null;
        }
    };
    /**
     * Filters an array of items and sets state. This method might be debounced in [[constructor]].
     *
     * @param items - The array of items to filter.
     * @param value - The search value to base the filtering on.
     */
    Searchable.prototype.filterAndSetState = function (items, value) {
        var predicate = this.props.predicate;
        this.setState({
            items: value !== '' ? Searchable.filter(items, value, predicate) : items,
        });
    };
    Searchable.defaultProps = {
        debounce: true,
        debounceDuration: 100,
        initialValue: '',
    };
    return Searchable;
}(react_1.Component));
exports.default = Searchable;
